plot2.aldmck <- function(obj, col=NULL, pch=NULL, cex=NULL, bw = .1, by=NULL, 
    bytype=c("juxtapose", "superpose"), leg.col=3, leg.cex=.8, ...){
    require(lattice)
    bt <- match.arg(bytype)
    tpg <- trellis.par.get()
    if(!is.null(col) & is.null(pch)){
        tpg$superpose.symbol$col <- col
    }
    if(!is.null(pch) & is.null(col)){
        tpg$superpose.symbol$pch <- pch
    }
    if(!is.null(pch) & !is.null(col)){
        eg <- expand.grid(pch, col)
        tpg$superpose.symbol$pch = eg[,1]
        tpg$superpose.symbol$col = as.character(eg[,2])
    }

    if(!is.null(cex)){
        tpg$superpose.symbol$cex <- cex
    }
    trellis.par.set(tpg)
    resps <- na.omit(obj$respondents)
    if(!is.null(by)){
        resps[[by]] <- eval(as.name(by), envir=.GlobalEnv)
        levs <- levels(resps[[by]])
        for(i in 1:length(levs)){
            levs[i] <- paste(levs[i], " (N = ", sum(resps[[by]] == levs[i]), ")", sep="")
        }
        levels(resps[[by]]) <- levs
    }
    stims <- obj$stimuli
    n <- paste("N = ", nrow(resps), sep="")
    mat <- suppressWarnings(cbind(c(names(stims), n), tpg$superpose.symbol$pch, tpg$superpose.symbol$col))
    mat <- mat[1:(length(stims)+1), ]
    mat[nrow(mat),c(2,3)] <- c(NA, NA)
    mykey <- list(space="top", columns=leg.col, points=list(pch=as.numeric(mat[,2]), col=mat[,3]), 
        text=list(mat[,1]), cex=leg.cex)
    mx <- min(resps$idealpt)
    ys <- rep(0, length(stims))
    if(is.null(by)){
        dens <- density(resps$idealpt, bw=bw)
        maxy <- max(dens$y)
    }
    else{
        dens <- by(resps$idealpt, list(resps[[by]]), function(z)density(z, bw=bw))
        maxy <- max(sapply(dens, function(res)max(res$y)))
    }
    yl <- c(0, maxy)
    if(!is.null(by) & bt == "juxtapose"){
        form <- as.formula(paste("~ idealpt | ", by, sep=""))
        p <- densityplot(form, data=resps, scales=list(relation="free"), bw = bw, ..., 
            panel = function(x,...){
                panel.abline(h=0, col="gray75", lty=3)
                panel.densityplot(x, pch=NA, col="black")
                panel.points(stims, ys, pch=tpg$superpose.symbol$pch, 
                    col=tpg$superpose.symbol$col, cex=tpg$superpose.symbol$cex)
            }, key=mykey)
    }
    else{
        form <- as.formula("~ idealpt")
        if(is.null(by)){
        p <- densityplot(form, data=resps, bw = bw, ..., 
            panel = function(x,...){
                panel.abline(h=0, col="gray75", lty=3)
                panel.densityplot(x, pch=NA, col="black")
                panel.points(stims, ys, pch=tpg$superpose.symbol$pch, 
                    col=tpg$superpose.symbol$col, cex=tpg$superpose.symbol$cex)
            }, key=mykey)
        }
    if(!is.null(by) & bt == "superpose"){
        tpg$superpose.line$col <- tpg$superpose.symbol$col
        trellis.par.set(tpg)
        p <- densityplot(form, groups = resps[[by]], data=resps, scales=list(relation="free"), bw = bw, ..., 
            panel = function(x,groups,...){
                panel.abline(h=0, col="gray75", lty=3)
                panel.superpose(x, groups=groups, panel.groups=panel.densityplot, pch=NA,...)
                panel.points(stims, ys, pch=tpg$superpose.symbol$pch, 
                    col=tpg$superpose.symbol$col, cex=tpg$superpose.symbol$cex)
            }, key=mykey)

        }
    }
    return(p)
}


boot.aldmck <- function(data, respondent=0, missing=NULL, polarity, verbose=FALSE, R=2500){
    boot.fun <- function(data, inds, polarity, missing, respondent, verbose){
    	assign(".inds", inds, envir=.GlobalEnv)
    	out <- aldmck(data[.inds,], polarity=polarity, respondent=respondent,
            missing=missing, verbose=verbose)
    	remove(".inds", envir=.GlobalEnv)
    	out$stimuli
    }
    res  <- boot(franceEES2009, boot.fun, R=R, polarity=polarity, missing=missing, respondent=respondent, verbose=verbose)
    class(res) <- "baldmck"
    return(res)
}

ci.aldmck <- function(obj, ci=c("bca", "perc", "norm")){
    type <- match.arg(ci)
    cis <- NULL
    for(i in 1:length(obj$t0)){
        cis <- rbind(cis, boot.cis <- boot.ci(out, type=type, index=i)[[4]])
    }
    cis <- cis[,(ncol(cis)-1):ncol(cis)]
    rownames(cis) <- names(obj$t0)
    cis <- cbind(obj$t0, cis)
    colnames(cis) <- c("idealpt", "lower", "upper")
    reslist <- list(ci = cis, boot.res=obj)
}




plot.baldmck <- function(obj,...){
    require(lattice)
    cis <- as.data.frame(obj$ci)
    xl <- range(c(cis))+ c(-1,1)*.05*diff(range(c(cis)))
    cis$rn <- as.factor(rownames(cis))
    cis$rn <- reorder(cis$rn, cis[,1])
    p <- dotplot(rn ~ idealpt, data=cis, xlim=xl, ...,
        lower = cis$lower, upper=cis$upper,
        panel = function(x,y,lower,upper,...){
            panel.segments(lower, y, upper, y, col="black")
            panel.xyplot(x,y,pch=16, col="black", ...)
        })
    return(p)
}


test.baldmck <- function(obj, level=.95, digits=3){
    mat <- obj$t
    colnames(mat) <- names(obj$t0)
    d <- combn(ncol(mat), 2)
    D <- matrix(0, nrow=ncol(mat), ncol=ncol(d))
    D[cbind(d[1,], 1:ncol(d))] <- -1
    D[cbind(d[2,], 1:ncol(d))] <- 1
    diffs <- mat %*% D
    colnames(diffs) <- apply(apply(d, 2, function(x)colnames(mat)[rev(x)]), 2, paste, collapse="-")
    diffci <- t(apply(diffs, 2, quantile, c(.025, .975)))
    meandiff <- c(obj$t0 %*% D)
    p1 <- data.frame(diff = meandiff, lower=diffci[,1], upper=diffci[,2])
    p1 <- round(p1, digits)
    sig <- ifelse(sign(p1[,1]) == sign(p1[,2]), "*", "")
    p1$sig <- sig
    return(p1)
}


plot.bb <- function(obj, data, by=NULL, points=TRUE, col = NULL, pch=NULL, rug=FALSE, 
    nv = FALSE, exp.factor=.6, xlim=NULL, ylim=NULL,...){
    x <- obj$individuals[[2]][,1]
    y <- obj$individuals[[2]][,2]
    if(!is.null(by)){
        group <- eval(as.name(by), envir=.GlobalEnv)
    }
    if(is.null(col)){
        if(is.null(by)){
            col <- "black"
        }
        else{
             col <- palette()[1:length(unique(group))]
        }
    }
    if(is.null(pch)){
        if(is.null(by)){
            pch <- 1
        }
        else{
             pch <- 1:length(unique(group))
        }
    }
    if(!is.null(by)){
        cols <- col[as.numeric(group)]
        pchs <- pch[as.numeric(group)]
    }
    if(is.null(xlim)){
        xl <- range(x, na.rm=T) + c(-1,1)*(.05 + .05*length(unique(group)))*diff(range(x, na.rm=T))
    }
    else{
        xl <- xlim
    }
    if(is.null(ylim)){
        yl <- range(y, na.rm=T) + c(-1,1)*(.05 + .05*length(unique(group)))*diff(range(y, na.rm=T))
    }
    else{
        yl <- ylim
    }
    if(points){
        plot(x, y, col=cols, pch=pchs, xlab = "First Dimension", ylab="Second Dimension", 
            xlim=xl, ylim=yl, ...)
    }
    if(!points){
        plot(x, y, col=cols, pch=pchs, xlab = "First Dimension", ylab="Second Dimension", 
            xlim=xl, ylim=yl, type="n", ...)
    }
    if(rug){
        ung <- unique(group)
        for(i in 1:length(unique(group))){
            rug(x[which(group == ung[i])], ticksize=.02, col=col[i], side=1, line=(-.5*(i-1)))
            rug(y[which(group == ung[i])], ticksize=.02, col=col[i], side=2, line=(-.5*(i-1)))
        }
    }
    if(nv){
        require(MASS)
        b <- NULL
        for(i in 1:ncol(data)){
            b <- rbind(b, coef(polr(as.factor(data[,i]) ~ x + y, method="probit")))
        }
        rownames(b) <- colnames(data)
        b <- t(apply(b, 1, function(x)x/sqrt(sum(x^2))))
    }
    
    arrows(0, 0, exp.factor*b[,1], exp.factor*b[,2], code=2, length=.1)
    text(exp.factor*b[,1], exp.factor*b[,2], rownames(b), pos=ifelse(b[,1] < 0, 2, 4), cex=.65)
}
